@@ -40,7 +40,7 @@ int _kbhit() {
 
 #include "resip/stack/InteropHelper.hxx"
 #include "recon/UserAgent.hxx"
-#include "recon/ReconSubsystem.hxx"
+#include "AppSubsystem.hxx"
 #include <recon/SipXHelper.hxx>
 
 #include <os/OsSysLog.h>
@@ -50,6 +50,7 @@ int _kbhit() {
 #include "MyMessageDecorator.hxx"
 #include "MyConversationManager.hxx"
 #include "B2BCallManager.hxx"
+#include "CDRFile.hxx"
 #include "RegistrationForwarder.hxx"
 
 #include <rutil/Log.hxx>
@@ -61,12 +62,13 @@ int _kbhit() {
 #include <resip/stack/HEPSipMessageLoggingHandler.hxx>
 #include <reflow/HEPRTCPEventLoggingHandler.hxx>
 
+using namespace reconserver;
 using namespace recon;
 using namespace resip;
 using namespace flowmanager;
 using namespace std;
 
-#define RESIPROCATE_SUBSYSTEM ReconSubsystem::RECON
+#define RESIPROCATE_SUBSYSTEM AppSubsystem::RECONSERVER
 
 void sleepSeconds(unsigned int seconds)
 {
@@ -758,31 +760,13 @@ void ReConServerProcess::processKeyboard(char input, MyConversationManager& myCo
 int 
 ReConServerProcess::main (int argc, char** argv)
 {
-#ifndef _WIN32
-   if ( signal( SIGPIPE, SIG_IGN) == SIG_ERR)
-   {
-      cerr << "Couldn't install signal handler for SIGPIPE" << endl;
-      exit(-1);
-   }
-#endif
+   installSignalHandler();
 
 #if defined(WIN32) && defined(_DEBUG) && defined(LEAK_CHECK) 
    resip::FindMemoryLeaks fml;
    {
 #endif
 
-   if ( signal( SIGINT, signalHandler ) == SIG_ERR )
-   {
-      cerr << "Couldn't install signal handler for SIGINT" << endl;
-      exit( -1 );
-   }
-
-   if ( signal( SIGTERM, signalHandler ) == SIG_ERR )
-   {
-      cerr << "Couldn't install signal handler for SIGTERM" << endl;
-      exit( -1 );
-   }
-
    Data defaultConfigFilename("reConServer.config");
    ReConServerConfig reConServerConfig;
    try
@@ -797,11 +781,11 @@ ReConServerProcess::main (int argc, char** argv)
 
    Data pidFile = reConServerConfig.getConfigData("PidFile", "", true);
    bool daemonize = reConServerConfig.getConfigBool("Daemonize", false);
-   bool keyboardInput = reConServerConfig.getConfigBool("KeyboardInput", !daemonize);
-   if(daemonize && keyboardInput)
+   mKeyboardInput = reConServerConfig.getConfigBool("KeyboardInput", !daemonize);
+   if(daemonize && mKeyboardInput)
    {
       ErrLog(<< "Ignoring KeyboardInput=true setting as we are running as a daemon");
-      keyboardInput = false;
+      mKeyboardInput = false;
    }
    setPidFile(pidFile);
    // Daemonize if necessary
@@ -841,6 +825,7 @@ ReConServerProcess::main (int argc, char** argv)
    Data loggingLevel = reConServerConfig.getConfigData("LoggingLevel", "INFO", true);
    Data loggingFilename = reConServerConfig.getConfigData("LogFilename", "reConServer.log", true);
    unsigned int loggingFileMaxLineCount = reConServerConfig.getConfigUnsignedLong("LogFileMaxLines", 50000);
+   Data cdrLogFilename = reConServerConfig.getConfigData("CDRLogFile", "", true);
    Data captureHost = reConServerConfig.getConfigData("CaptureHost", "");
    int capturePort = reConServerConfig.getConfigInt("CapturePort", 9060);
    int captureAgentID = reConServerConfig.getConfigInt("CaptureAgentID", 2002);
@@ -926,7 +911,7 @@ ReConServerProcess::main (int argc, char** argv)
    InfoLog( << "  Log Level = " << loggingLevel);
    InfoLog( << "  Log Filename = " << loggingFilename);
    InfoLog( << "  Daemonize = " << (daemonize ? "true" : "false"));
-   InfoLog( << "  KeyboardInput = " << (keyboardInput ? "true" : "false"));
+   InfoLog( << "  KeyboardInput = " << (mKeyboardInput ? "true" : "false"));
    InfoLog( << "  PidFile = " << pidFile);
    InfoLog( << "  Run as user = " << runAsUser);
    InfoLog( << "  Run as group = " << runAsGroup);
@@ -938,6 +923,25 @@ ReConServerProcess::main (int argc, char** argv)
 
    SharedPtr<UserAgentMasterProfile> profile(new UserAgentMasterProfile);
 
+   Data certPath;
+   reConServerConfig.getConfigValue("CertificatePath", certPath);
+   if(!certPath.empty())
+   {
+      profile->certPath() = certPath;
+   }
+   Data caDir;
+   reConServerConfig.getConfigValue("CADirectory", caDir);
+   if(!caDir.empty())
+   {
+      profile->rootCertDirectories().push_back(caDir);
+   }
+   Data caFile;
+   reConServerConfig.getConfigValue("CAFile", caFile);
+   if(!caFile.empty())
+   {
+      profile->rootCertBundles().push_back(caFile);
+   }
+
    if(!captureHost.empty())
    {
       SharedPtr<HepAgent> agent(new HepAgent(captureHost, capturePort, captureAgentID));
@@ -1291,21 +1295,27 @@ ReConServerProcess::main (int argc, char** argv)
    // Create ConverationManager and UserAgent
    //////////////////////////////////////////////////////////////////////////////
    {
-      std::auto_ptr<MyConversationManager> conversationManager;
       switch(application)
       {
          case ReConServerConfig::None:
-            conversationManager.reset(new MyConversationManager(localAudioEnabled, mediaInterfaceMode, defaultSampleRate, maximumSampleRate, autoAnswerEnabled));
+            mConversationManager.reset(new MyConversationManager(localAudioEnabled, mediaInterfaceMode, defaultSampleRate, maximumSampleRate, autoAnswerEnabled));
             break;
          case ReConServerConfig::B2BUA:
-            conversationManager.reset(new B2BCallManager(mediaInterfaceMode, defaultSampleRate, maximumSampleRate, reConServerConfig));
+            {
+               SharedPtr<B2BCallLogger> b2bCallLogger;
+               if(!cdrLogFilename.empty())
+               {
+                  b2bCallLogger.reset(new CDRFile(cdrLogFilename));
+               }
+               mConversationManager.reset(new B2BCallManager(mediaInterfaceMode, defaultSampleRate, maximumSampleRate, reConServerConfig, b2bCallLogger));
+            }
             break;
          default:
             assert(0);
       }
-      MyUserAgent ua(reConServerConfig, conversationManager.get(), profile);
-      conversationManager->buildSessionCapabilities(address, numCodecIds, codecIds, conversationProfile->sessionCaps());
-      ua.addConversationProfile(conversationProfile);
+      mUserAgent.reset(new MyUserAgent(reConServerConfig, mConversationManager.get(), profile));
+      mConversationManager->buildSessionCapabilities(address, numCodecIds, codecIds, conversationProfile->sessionCaps());
+      mUserAgent->addConversationProfile(conversationProfile);
 
       if(application == ReConServerConfig::B2BUA)
       {
@@ -1314,11 +1324,20 @@ ReConServerProcess::main (int argc, char** argv)
          if(!internalMediaAddress.empty())
          {
             SharedPtr<ConversationProfile> internalProfile(new ConversationProfile(conversationProfile));
+            Data b2BUANextHop = reConServerConfig.getConfigData("B2BUANextHop", "", true);
+            if(b2BUANextHop.empty())
+            {
+               CritLog(<<"Please specify B2BUANextHop");
+               throw ConfigParse::Exception("Please specify B2BUANextHop", __FILE__, __LINE__);
+            }
+            NameAddrs route;
+            route.push_front(NameAddr(b2BUANextHop));
+            internalProfile->setServiceRoute(route);
             internalProfile->secureMediaMode() = reConServerConfig.getConfigSecureMediaMode("B2BUAInternalSecureMediaMode", secureMediaMode);
             internalProfile->setDefaultFrom(uri);
             internalProfile->setDigestCredential(uri.uri().host(), uri.uri().user(), password);
-            conversationManager->buildSessionCapabilities(internalMediaAddress, numCodecIds, codecIds, internalProfile->sessionCaps());
-            ua.addConversationProfile(internalProfile, false);
+            mConversationManager->buildSessionCapabilities(internalMediaAddress, numCodecIds, codecIds, internalProfile->sessionCaps());
+            mUserAgent->addConversationProfile(internalProfile, false);
          }
          else
          {
@@ -1330,10 +1349,10 @@ ReConServerProcess::main (int argc, char** argv)
       // Startup and run...
       //////////////////////////////////////////////////////////////////////////////
 
-      ua.startup();
-      conversationManager->startup();
+      mUserAgent->startup();
+      mConversationManager->startup();
 
-      //ua.createSubscription("message-summary", uri, 120, Mime("application", "simple-message-summary")); // thread safe
+      //mUserAgent->createSubscription("message-summary", uri, 120, Mime("application", "simple-message-summary")); // thread safe
 
       // Drop privileges (can do this now that sockets are bound)
       if(!runAsUser.empty())
@@ -1342,29 +1361,9 @@ ReConServerProcess::main (int argc, char** argv)
          dropPrivileges(runAsUser, runAsGroup);
       }
 
-      int input;
-      while(true)
-      {
-         ua.process(50);
-         if(keyboardInput)
-         {
-            while(_kbhit() != 0)
-            {
-#ifdef WIN32
-               input = _getch();
-               processKeyboard(input, *conversationManager, ua);
-#else
-               input = fgetc(stdin);
-               fflush(stdin);
-               //cout << "input: " << input << endl;
-               processKeyboard(input, *conversationManager, ua);
-#endif
-            }
-         }
-         if(finished) break;
-      }
+      mainLoop();
 
-      ua.shutdown();
+      mUserAgent->shutdown();
    }
    InfoLog(<< "reConServer is shutdown.");
    OsSysLog::shutdown();
@@ -1375,6 +1374,39 @@ ReConServerProcess::main (int argc, char** argv)
 #endif
 }
 
+void
+ReConServerProcess::doWait()
+{
+   mUserAgent->process(50);
+}
+
+void
+ReConServerProcess::onLoop()
+{
+   if(mKeyboardInput)
+   {
+      int input;
+      while(_kbhit() != 0)
+      {
+#ifdef WIN32
+         input = _getch();
+         processKeyboard(input, *mConversationManager, *mUserAgent);
+#else
+         input = fgetc(stdin);
+         fflush(stdin);
+         //cout << "input: " << input << endl;
+         processKeyboard(input, *mConversationManager, *mUserAgent);
+#endif
+      }
+   }
+}
+
+void
+ReConServerProcess::onReload()
+{
+   StackLog(<<"ReConServerProcess::onReload invoked");
+}
+
 
 /* ====================================================================
 
